# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: UDFImages.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0fUDFImages.proto\x12\x08protocol\"9\n\x07Wrapper\x12\x1e\n\x05image\x18\x01 \x03(\x0b\x32\x0f.protocol.Image\x12\x0e\n\x06params\x18\x02 \x01(\t\"y\n\x05Image\x12/\n\x08\x62\x61ndData\x18\x01 \x03(\x0b\x32\x1d.protocol.Image.BandDataEntry\x1a?\n\rBandDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1d\n\x05value\x18\x02 \x01(\x0b\x32\x0e.protocol.Band:\x02\x38\x01\"[\n\x04\x42\x61nd\x12\x10\n\x08tileData\x18\x01 \x01(\x0c\x12\x10\n\x08\x64\x61taType\x18\x02 \x01(\t\x12\x13\n\x0bnodataValue\x18\x03 \x01(\t\x12\x0c\n\x04\x63ols\x18\x04 \x01(\x05\x12\x0c\n\x04rows\x18\x05 \x01(\x05\x42\x33\n\x1c\x63om.alibaba.aie.engine.modelB\tUDFImagesZ\x08protocolb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'UDFImages_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\034com.alibaba.aie.engine.modelB\tUDFImagesZ\010protocol'
  _IMAGE_BANDDATAENTRY._options = None
  _IMAGE_BANDDATAENTRY._serialized_options = b'8\001'
  _globals['_WRAPPER']._serialized_start=29
  _globals['_WRAPPER']._serialized_end=86
  _globals['_IMAGE']._serialized_start=88
  _globals['_IMAGE']._serialized_end=209
  _globals['_IMAGE_BANDDATAENTRY']._serialized_start=146
  _globals['_IMAGE_BANDDATAENTRY']._serialized_end=209
  _globals['_BAND']._serialized_start=211
  _globals['_BAND']._serialized_end=302
# @@protoc_insertion_point(module_scope)
